#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include "projective_primitives.h"


/**************************************************************************
 * First we have the code for the CP2Point class                          *
 **************************************************************************/

static int point_id = 1;

/* a default constructor */
CP2Point::CP2Point(void)
{
	p2pos[0] = 0.0;
	p2pos[1] = 0.0;
	p2pos[2] = 0.0;
	id = point_id++;

	set_dr_flags(DR_P2_POINT);
}

/* constructor */
CP2Point::CP2Point(double x1, double x2, double x3)
{
	set_val(x1, x2, x3);
	id = point_id++;

	set_dr_flags(DR_P2_POINT);
}

/* another constructor */
CP2Point::CP2Point(double *pos)
{
	set_val(pos);
	id = point_id++;

	set_dr_flags(DR_P2_POINT);
}

void CP2Point::set_val(double x1, double x2, double x3)
{
	p2pos[0] = x1;
	p2pos[1] = x2;
	p2pos[2] = x3;
}

void CP2Point::set_val(double *pos)
{
	p2pos[0] = pos[0];
	p2pos[1] = pos[1];
	p2pos[2] = pos[2];
}

void CP2Point::get_val(double *ret)
{
	ret[0] = p2pos[0];
	ret[1] = p2pos[1];
	ret[2] = p2pos[2];
}

int CP2Point::get_id()
{
	return id;
}

/* Draw a cube at the point pos of the size len 
 * Stolen from Jacks code to draw the axes
 */
void CP2Point::draw_cube(double *pos, double len)
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();					// Save current matrix
	glTranslated(pos[0], pos[1], pos[2]); /* translate it to the right place */
	glScaled(len, len, len);	// scale to proper size
	glBegin(GL_QUADS);
	glVertex3f( 1.0f, 1.0f, 1.0f);	// +x
	glVertex3f( 1.0f,-1.0f, 1.0f);	
	glVertex3f( 1.0f,-1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f,-1.0f);

	glVertex3f( 1.0f, 1.0f, 1.0f);	// +y
	glVertex3f( 1.0f, 1.0f,-1.0f);
	glVertex3f(-1.0f, 1.0f,-1.0f);
	glVertex3f(-1.0f, 1.0f, 1.0f);

	glVertex3f( 1.0f, 1.0f, 1.0f);	// +z
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f,-1.0f, 1.0f);
	glVertex3f( 1.0f,-1.0f, 1.0f);

	glVertex3f(-1.0f,-1.0f,-1.0f);	// -x
	glVertex3f(-1.0f,-1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f,-1.0f);

	glVertex3f(-1.0f,-1.0f,-1.0f);	// -y
	glVertex3f( 1.0f,-1.0f,-1.0f);
	glVertex3f( 1.0f,-1.0f, 1.0f);
	glVertex3f(-1.0f,-1.0f, 1.0f);

	glVertex3f(-1.0f,-1.0f,-1.0f);	// -z
	glVertex3f(-1.0f, 1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f,-1.0f);
	glVertex3f( 1.0f,-1.0f,-1.0f);
	glEnd();
	glPopMatrix();					// Restore original matrix.
}

/* Draw only the P2 points for selection */
void CP2Point::draw_select()
{
	glDisable(GL_DITHER);
	/* first switch off lighting and blending */
	glPushAttrib(GL_LIGHTING_BIT);			// save current lighting state
	glPushAttrib(GL_COLOR_BUFFER_BIT);		// and transparency state
	glDisable(GL_LIGHTING);					// turn off lights for axes,
	glDisable(GL_BLEND);					// turn off transparency.

	if(dr_flags & DR_P2_POINT) {
		/* Draw the point in the (x,y,-1) plane */
		double p2[3];

		if((p2pos[2] < DELTA) && (p2pos[2] > -DELTA)) {
			/* This represents an infinite point. Simply ignore it! */
		} else {
			p2[0] = -p2pos[0]/p2pos[2];
			p2[1] = -p2pos[1]/p2pos[2];
			p2[2] = -1;

			glColor3ub(0, 0, id); /* use the id for the color */
			draw_cube(p2, 0.01);
		}
	}

	glPopAttrib();						// restore original transparency state.
	glPopAttrib();						// restore original lighting state.
}

/* Set drawing flags */
void CP2Point::set_dr_flags(unsigned char drf)
{
	dr_flags = drf;
}

/* Performs the actual drawing operations. optionally give the cube's color */
void CP2Point::draw_me(float r, float g, float b)
{
	/* first switch off lighting and blending */
	glPushAttrib(GL_LIGHTING_BIT);			// save current lighting state
	glPushAttrib(GL_COLOR_BUFFER_BIT);		// and transparency state
	glDisable(GL_LIGHTING);					// turn off lights for axes,
	glDisable(GL_BLEND);					// turn off transparency.

	/* We do the expected things as per the instructions stored for us in the flags */
	if(dr_flags & DR_3D_POINT) {
		/* Draw the 3d point as a small cube */
		if(dr_flags & DR_RAY)
			glColor3f(1.0, 1.0, 1.0); /* draw of a different color if ray is also to be drawn */
		else
			glColor3f(r, g, b);

		draw_cube(p2pos, 0.01);
	}

	if(dr_flags & DR_P2_POINT) {
		/* Draw the point in the (x,y,-1) plane */
		double p2[3];

		if((p2pos[2] < DELTA) && (p2pos[2] > -DELTA)) {
			/* This represents an infinite point. Simply ignore it! */
		} else {
			p2[0] = -p2pos[0]/p2pos[2];
			p2[1] = -p2pos[1]/p2pos[2];
			p2[2] = -1;

			if(dr_flags & DR_RAY)
				glColor3f(1.0, 1.0, 1.0); /* draw of a different color if ray is also to be drawn */
			else
				glColor3f(r, g, b);

			draw_cube(p2, 0.01);
		}
	}

	if(dr_flags & DR_RAY) {
		/* Draw the ray for the point */
		double norm, xx, yy, zz;

		/* first normalize the p2pos[] */
		norm = sqrt(p2pos[0]*p2pos[0] + p2pos[1]*p2pos[1] + p2pos[2]*p2pos[2]);

		if(norm < DELTA) {
			/* dont draw the ray! */
		} else {
			glColor3f(0.5, 0.5, 1.0);
			xx = RAY_SZ * p2pos[0] / norm;
			yy = RAY_SZ * p2pos[1] / norm;
			zz = RAY_SZ * p2pos[2] / norm;
			glBegin(GL_LINES);
				glVertex3d(xx, yy, zz);
				glVertex3d(0, 0, 0);
			glEnd();
		}
	}

	glPopAttrib();						// restore original transparency state.
	glPopAttrib();						// restore original lighting state.
}

/**************************************************************************
 * End of the CP2Point class                                              *
 **************************************************************************/



/**************************************************************************
 * Code for the CP2Line class                                             *
 **************************************************************************/

/* a default constructor */
CP2Line::CP2Line(void)
{
	line[0] = 0.0;
	line[1] = 0.0;
	line[2] = 0.0;

	set_dr_flags(DR_LINE | DR_PLANE);
}

/* constructor */
CP2Line::CP2Line(double x1, double x2, double x3)
{
	set_val(x1, x2, x3);

	set_dr_flags(DR_LINE | DR_PLANE);
}

/* another constructor */
CP2Line::CP2Line(double *p)
{
	set_val(p);

	set_dr_flags(DR_LINE | DR_PLANE);
}

/* yet another constructor. line passes thru these points */
CP2Line::CP2Line(CP2Point p1, CP2Point p2)
{
	set_val(p1, p2);

	set_dr_flags(DR_LINE | DR_PLANE);
}

void CP2Line::set_val(double x1, double x2, double x3)
{
	line[0] = x1;
	line[1] = x2;
	line[2] = x3;
}

void CP2Line::set_val(double *p)
{
	line[0] = p[0];
	line[1] = p[1];
	line[2] = p[2];
}

/* make the line pass through these points */
void CP2Line::set_val(CP2Point p1, CP2Point p2)
{
	/* line thru two points is l = x * x' */
	double pt1[3], pt2[3];

	p1.get_val(pt1);
	p2.get_val(pt2);

	line[0] = pt1[1]*pt2[2] - pt1[2]*pt2[1];
	line[1] = pt1[2]*pt2[0] - pt1[0]*pt2[2];
	line[2] = pt1[0]*pt2[1] - pt1[1]*pt2[0];
}

void CP2Line::get_val(double *ret)
{
	ret[0] = line[0];
	ret[1] = line[1];
	ret[2] = line[2];
}


/* Set drawing flags */
void CP2Line::set_dr_flags(unsigned char drf)
{
	dr_flags = drf;
}

/* Performs the actual drawing operations */
void CP2Line::draw_me(void)
{
	double x1, x2, y1, y2;

	/* first switch off lighting and blending */
	glPushAttrib(GL_LIGHTING_BIT);			// save current lighting state
	glPushAttrib(GL_COLOR_BUFFER_BIT);		// and transparency state
	glDisable(GL_LIGHTING);					// turn off lights for axes,
	glDisable(GL_BLEND);					// turn off transparency.

	/* First check if both A and B are 0 */
	if((line[0] < DELTA) && (line[0] > -DELTA)
		&& (line[1] < DELTA) && (line[1] > -DELTA) ) {
		/* Sorry, we cant draw the line or the plane */
		glPopAttrib();						// restore original transparency state.
		glPopAttrib();						// restore original lighting state.

		return;
	} else {
		if((line[0] < DELTA) && (line[0] > -DELTA)) {
			/* A = 0. line is parallel to x axis */
			double y = -line[2]/line[1];

			x1 = -RAY_SZ;
			y1 = y;
			x2 = RAY_SZ;
			y2 = y;
		} else if((line[1] < DELTA) && (line[1] > -DELTA)) {
			/* B = 0. line is parallel to x axis */
			double x = -line[2]/line[0];
			x1 = x;
			y1 = -RAY_SZ;
			x2 = x;
			y2 = RAY_SZ;
		} else {
			/* the most common case. Neither A nor B are 0.
			 * Find two points on the line. First put x=1, then x=-1
			 */
			x1 = 1.5;
			y1 = -(line[2] + x1*line[0])/line[1];

			x2 = -1.5;
			y2 = -(line[2] + x2*line[0])/line[1];


		}
	}

	/* We do the expected things as per the instructions stored for us in the flags */
	if(dr_flags & DR_LINE) {
		/* Draw the line on the (x,y,1) plane */
		glColor3f(1.0, 1.0, 1.0);

		/* Now join the two points! */
		glBegin(GL_LINES);
			glVertex3d(x1, y1, 1);
			glVertex3d(x2, y2, 1);
		glEnd();
	}

	if(dr_flags & DR_PLANE) {
		/* Draw a translucent plane corresponding to the line */

		glEnable(GL_BLEND);
		glDisable(GL_DEPTH_TEST);

		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glColor4d(0.0, 1.0, 1.0, 0.15);

		glBegin(GL_TRIANGLES);
			glVertex3d(0, 0, 0);
			glVertex3d(x1*P_MULT, y1*P_MULT, P_MULT);
			glVertex3d(x2*P_MULT, y2*P_MULT, P_MULT);
		glEnd();

		glEnable(GL_DEPTH_TEST);
		glDisable(GL_BLEND);
	}

	glPopAttrib();						// restore original transparency state.
	glPopAttrib();						// restore original lighting state.
}

/**************************************************************************
 * End of the CP2Line class                                               *
 **************************************************************************/



/**************************************************************************
 * Code for the CP2H class                                                *
 **************************************************************************/

/* constructor */
CP2H::CP2H(double h11, double h12, double h13, double h21, double h22,
		   double h23, double h31, double h32, double h33)
{
	set_val(h11, h12, h13, h21, h22, h23, h31, h32, h33);
}

/* another constructor */
CP2H::CP2H(double *h_mat)
{
	set_val(h_mat);
}

void CP2H::set_val(double h11, double h12, double h13, double h21, double h22,
		   double h23, double h31, double h32, double h33)
{
	h[0][0] = h11;
	h[0][1] = h12;
	h[0][2] = h13;

	h[1][0] = h21;
	h[1][1] = h22;
	h[1][2] = h23;

	h[2][0] = h31;
	h[2][1] = h32;
	h[2][2] = h33;
}

void CP2H::set_val(double *h_mat)
{
	memcpy(h, h_mat, sizeof(double) * P2_DIMS * P2_DIMS);
}

double CP2H::get_val(int x, int y)
{
	return h[x][y];
}

void CP2H::get_val(double *ret)
{
	memcpy(ret, h, sizeof(double) * P2_DIMS * P2_DIMS);
}

/* transform the point (x' = H x) */
void CP2H::transform_point(CP2Point &point)
{
	double xf_pt[P2_DIMS], pt[P2_DIMS];

	point.get_val(pt);

	/* apply the transform */
	xf_pt[0] = h[0][0] * pt[0] + h[0][1] * pt[1] + h[0][2] * pt[2];
	xf_pt[1] = h[1][0] * pt[0] + h[1][1] * pt[1] + h[1][2] * pt[2];
	xf_pt[2] = h[2][0] * pt[0] + h[2][1] * pt[1] + h[2][2] * pt[2];

	point.set_val(xf_pt);
}

/* transform the line (l' = H^(-T) l). returns 0 on success, -1 if cant invert the matrix. */
int CP2H::transform_line(CP2Line &line)
{
	double h_it[P2_DIMS][P2_DIMS];
	double xf_l[P2_DIMS], l[P2_DIMS];
	double det;

	/* First find the inverse to H
	 * Currently has dirty code (for 3x3 matrix only)
	 */
	det = h[0][0] * (h[1][1]*h[2][2] - h[2][1]*h[1][2])
		- h[0][1] * (h[1][0]*h[2][2] - h[2][0]*h[1][2])
		+ h[0][2] * (h[1][0]*h[2][1] - h[2][0]*h[1][1]);

	/* determinant == 0? */
	if((det<DELTA) && (det>-DELTA))
		return -1;

	/* okay, find the inverse transpose! */
	h_it[0][0] = (h[1][1]*h[2][2] - h[2][1]*h[1][2])/det;
	h_it[0][1] = (h[1][2]*h[2][0] - h[2][2]*h[1][0])/det;
	h_it[0][2] = (h[1][0]*h[2][1] - h[2][0]*h[1][1])/det;

	h_it[1][0] = (h[0][2]*h[2][1] - h[2][2]*h[0][1])/det;
	h_it[1][1] = (h[0][0]*h[2][2] - h[2][0]*h[0][2])/det;
	h_it[1][2] = (h[0][1]*h[2][0] - h[2][1]*h[0][0])/det;

	h_it[2][0] = (h[0][1]*h[1][2] - h[1][1]*h[0][2])/det;
	h_it[2][1] = (h[0][2]*h[1][0] - h[1][2]*h[0][0])/det;
	h_it[2][2] = (h[0][0]*h[1][1] - h[1][0]*h[0][1])/det;


	/* now do l' = h_it * l */
	line.get_val(l);

	xf_l[0] = h_it[0][0] * l[0] + h_it[0][1] * l[1] + h_it[0][2] * l[2];
	xf_l[1] = h_it[1][0] * l[0] + h_it[1][1] * l[1] + h_it[1][2] * l[2];
	xf_l[2]	= h_it[2][0] * l[0] + h_it[2][1] * l[1] + h_it[2][2] * l[2];

	line.set_val(xf_l);

	return 0;
}


CP2H CP2H::inverse(void)
{
	double h_it[P2_DIMS][P2_DIMS];
	double det;
	CP2H Hinv;

	/* First find the inverse to H
	 * Currently has dirty code (for 3x3 matrix only)
	 */
	det = h[0][0] * (h[1][1]*h[2][2] - h[2][1]*h[1][2])
		- h[0][1] * (h[1][0]*h[2][2] - h[2][0]*h[1][2])
		+ h[0][2] * (h[1][0]*h[2][1] - h[2][0]*h[1][1]);

	/* determinant == 0? */
	if((det<DELTA) && (det>-DELTA))
		return Hinv;

	/* okay, find the inverse transpose! */
	h_it[0][0] = (h[1][1]*h[2][2] - h[2][1]*h[1][2])/det;
	h_it[1][0] = (h[1][2]*h[2][0] - h[2][2]*h[1][0])/det;
	h_it[2][0] = (h[1][0]*h[2][1] - h[2][0]*h[1][1])/det;

	h_it[0][1] = (h[0][2]*h[2][1] - h[2][2]*h[0][1])/det;
	h_it[1][1] = (h[0][0]*h[2][2] - h[2][0]*h[0][2])/det;
	h_it[2][1] = (h[0][1]*h[2][0] - h[2][1]*h[0][0])/det;

	h_it[0][2] = (h[0][1]*h[1][2] - h[1][1]*h[0][2])/det;
	h_it[1][2] = (h[0][2]*h[1][0] - h[1][2]*h[0][0])/det;
	h_it[2][2] = (h[0][0]*h[1][1] - h[1][0]*h[0][1])/det;

	Hinv.set_val(&h_it[0][0]);

	return Hinv;
}