#ifndef _PROJECTIVE_PRIMITIVES_H_
#define _PROJECTIVE_PRIMITIVES_H_

#define DELTA	10E-3	/* very tiny number */
#define RAY_SZ	5		/* length of the ray */
#define P_MULT	1.5		/* Size of the plane */

#define P2_DIMS	3		/* number of coordinates for the projective geometry. P2 has 3 etc. */

/* Flags for drawing in different ways */
#define DR_NULL		0	/* Do not draw */
#define DR_3D_POINT	1	/* Draw a 3d point */
#define DR_P2_POINT	2	/* Draw the point in the (x,y,-1) plane */
#define DR_RAY		4	/* Draw the ray for the point */
#define DR_LINE		8	/* Draw the line in (x,y,1) plane) */
#define DR_PLANE	16	/* Draw the plane corresponding to the line */

class CP2Point
{
private:
	double p2pos[P2_DIMS]; /* point description in projective (x1, x2, x3) */
	unsigned char dr_flags;
	int id; /* point id, used for picking */

	void draw_cube(double *pos, double len); /* Makes a cube at pos */

public:
	/* constructors */
	CP2Point(void);
	CP2Point(double x1, double x2, double x3);
	CP2Point(double *);
	
	/* get and set stuff */
	void set_val(double x1, double x2, double x3);
	void set_val(double *);
	void get_val(double *ret);
	int get_id();

	void set_dr_flags(unsigned char drf);
	void draw_me(float r=1.0, float g=0.0, float b=1.0);
	void draw_select(); /* Draw only the P2 points for selection */
};
	
class CP2Line
{
private:
	double line[P2_DIMS]; /* line description (a, b, c) */
	unsigned char dr_flags;

public:
	/* constructors */
	CP2Line(void);
	CP2Line(double x1, double x2, double x3);
	CP2Line(double *);
	CP2Line(CP2Point p1, CP2Point p2); /* line passes thru these points */

	/* get and set stuff */
	void set_val(double x1, double x2, double x3);
	void set_val(double *);
	void set_val(CP2Point p1, CP2Point p2); /* make the line pass through these points */
	void get_val(double *ret);

	void set_dr_flags(unsigned char drf);
	void draw_me(void);
};


class CP2H
{
private:
	double h[P2_DIMS][P2_DIMS];

public:
	/* The constructors */
	CP2H(void) {};
	CP2H(double h11, double h12, double h13,
		 double h21, double h22, double h23,
		 double h31, double h32, double h33);
	CP2H(double *h_mat);

	/* Set and get specific values */
	void set_val(double h11, double h12, double h13,
				 double h21, double h22, double h23,
				 double h31, double h32, double h33);
	void set_val(double *h_mat);
	double get_val(int x, int y);
	void get_val(double *ret);

	/* transformation of point and line */
	void transform_point(CP2Point &point);
	int transform_line(CP2Line &line); /* returns -1 if cant invert the matrix */
	CP2H inverse(void);
};

#endif