//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
//
// NOTE!!! relies on quikGL class!!!
//
// quikMeshImg.h: interface for the CquikMeshImg class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_QUIKMESHIMG_H__FE0F53A2_3C3D_4B3D_8B0A_FA42D7EC4B60__INCLUDED_)
#define AFX_QUIKMESHIMG_H__FE0F53A2_3C3D_4B3D_8B0A_FA42D7EC4B60__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "projective_primitives.h"

#define NUM_PTS 4

/* type of selected point */
#define SEL_INPUT 0
#define SEL_OUTPUT 1

class CquikBMP
//==============================================================================
// Class to read and write an image in Windows' strange 'BMP' file format.
// The 'm_pImg' member points to a single RGB image buffer, used for both
// reading and writing BMP images. 
// Currently only reads/writes 24-bit RGB images without compression.
{
private:
	BITMAPFILEHEADER m_BMPone;				// 1st & (ptr to) 2nd structure at
	BITMAPINFO		 *m_pBMPtwo;			// the start of every BMP file.
	int m_xsize,m_ysize;					// image size given in pixels.

	BYTE *m_pImg;							// Dynamically alloc'd image buffer.
											// (Set to NULL if no memory taken)
											// holds BYTE-sized, RGB triplets
											// BUT (like BMP format) scanlines
											// are multiple-of-4 bytes each.
	int m_linesize;							// Size of a scanline in bytes;


public:										// All fcns return FALSE on error.
	CquikBMP();								// Default constructor/destructor
	~CquikBMP();
	void wipe();							// discard any current contents.

	int getXsize(void){return m_xsize;};		// Read image size;
	int getYsize(void){return m_ysize;};
	int getLinesize(void){return m_linesize;};
	BOOL sizer(int Xsize, int Ysize);		// Re-size image buffer to prepare
											// to write into it. Any size change
											// will discard buffer's contents.
	BOOL fileRead(OPENFILENAME *pName);		// Open, read, & close named file.
											// (Discards any existing contents)
	BOOL fileWrite(OPENFILENAME *pName);	// Open, write, & close named file
											// using current buffer contents.

	BYTE getPixel(int x, int y, int z);		// Read image buffer; z=0 for blue,
											// z=1 for green, z=2 for red.
	BYTE putPixel(int x, int y, int z, BYTE val);// Set an image buffer value
	BYTE* getPtr(int x, int y);				// Return pointer to image buffer
											// at location x,y; pixels stored in
											// BGR (not RGB) order (!).
};

class CquikPLY
//==============================================================================
// Class to read and write a 3D polygonal mesh in Greg Turk's 'PLY' format.
// The m_pV, m_pN, m_pT members point to buffers for vertices, normals, and
// texture coords respectively, used for both reading and writing PLY files.
{
};

class CquikMeshImg  
//==============================================================================
// 3D distortable image; holds a rectangular grid of vertices and a texture map
// image that is applied to it.  Initially a flat 'billboard' placed in 3D, 
// changing vertex positions in grid stretches to any 3D shape.  
{
private:
	GLfloat *m_pNorm, *m_pTexy, *m_pVert;	// pointers to dyn. alloc'd array 
											// of normal vectors, texture coords
											// and vertices for the mesh.
	float m_yTexmax, m_xTexmax;				// Largest texture-map coords that
											// are within the BMP image.

	void setupTexParams(void);				// Setup openGL texture map state 
											// to fit new/changed texture image.

	/* ankit: some points and lines for testing */
	CP2Point pt[2][NUM_PTS];		/* input(0), and output(1) points */
	CP2Line l1, l2;					/* two lines */
	CP2H m_H, m_Hnew;				/* The H matrices */
	int selected_type;				/* type of selected point (inp/out) */
	int selected_point;				/* index to the selected point, -1 => none selected */

public:
	CquikMeshImg();							// Constructor, initializer
	virtual ~CquikMeshImg();				// Destructor (default)

											// Texture Image:..................
	CquikBMP m_BMP;							// BMP-format image we'll use.
	GLint m_Txsize,m_Tysize,m_Tzsize;		// # of pixels, # of colors/pix
											// (MUST be 2^n x 2^n)
	GLubyte *m_pTimg;						// Ptr to the texture image
											// Mesh:..........................
	GLint m_Mxsize,m_Mysize;				// # of vertices in the mesh.
		
	void whatsup(void);						// error trap for debugging.
	BOOL fileSave(OPENFILENAME *pName);		// Save current texture map to file
											// <*pName>.bmp, and mesh geometry 
											// to <*pName>.ply
	BOOL fileLoad(OPENFILENAME *pName);		// Load texture map from file 
											// <*pName>.bmp and save mesh geom. 
											// from <*pName>.ply
	BOOL sizerTex(int Ximg,int Yimg,int Zimg);// Set texture map size: CAUTION
											// power-of-two sizes only! Any size
											// change discards current contents.
	BOOL sizerMesh(int Xmesh, int Ymesh);	// Set # of vertices in mesh. Size
											//changes discard current positions.

	void wipeTex(void);						// Discard any contents, release
											// all texture memory allocated.
	void wipeMesh(void);					// Discard any contents, release
											// all mesh memory allocated.
	void makeTestImgMeshDisplayList(void);	// Generate a test pattern texture.				
	void makeTestImgMesh(void);				// Generate a test pattern texture.
	void drawMe(void);						// call this only from within 
											// CquikGL::drawOpenGL() only.
	void meshWrinkle(float zoffset);		// Offset alternating mesh verts.			
	void error_GL(char *label);				// Report last openGL error, if any
											//(same as quikGL::errorGL())

	void select_point(int x, int y);		/* select the point at (x, y) screen coords */
	int get_selected();						/* return the selected point index */
	void set_selected(int type, int index);	/* set selectd point index */
	void move_selected_point(double dx1, double dx2, double dx3);	/* move the selected point */

	void interesting_H(void);				/* basically copy m_Hnew to m_H */
	void reset_H(void);						/* reset the H matrix to identity */
	void DLTsolver(void);					/* Solve DLT using pt[0] as input, and pt[1]
											   as output. Resulting H in m_Hnew */
};

#endif // !defined(AFX_QUIKMESHIMG_H__FE0F53A2_3C3D_4B3D_8B0A_FA42D7EC4B60__INCLUDED_)
